' C328 Camera Interface Program for PSM (Propeller System Module)
' Copyright 2009 Rayslogic.com, LLC

CON  'Camera Pin Assignments
  'using a couple of the 8 free Prop pins for the camera
  C328_TX= 18  '(yellow wire) - Camera1 TX (Yellow) [Prop RX]
  C328_RX= 17  '(green wire)    Camera1 RX (Green)  [Prop TX] 

CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000


OBJ
  lcd : "PSM_DisplayDriverC328"  'Controls the LCD and high-speed serial receiver
  sd  : "PSM_SdCardDriver"   'Controls the SD card 
  i2c : "PSM_i2cDriver"  'for AD7999 (used to read data from the 3-axis accelerometer, and extra ADC input)
                         'and AD5337 (used to set LCD brightness, and extra DAC output)
  ser : "JDCogSerial"  'using a faster serial interface 
  num : "numbers"  'Used to turn integers into formatted strings
  


VAR
  byte PicBuf[506]   'storage for jpg data packets
  long foregroundcolor,backgroundcolor  

    
PUB start|i
  'set default LCD colors
  backgroundcolor:=lcd.RGB(0,0,50) 'dark blue
  foregroundcolor:=lcd.RGB(255,255,0) 'yellow
  'initialize the LCD
  lcd.start(0)
  'turn on the LCD's LED backlight
  i2c.SetDAC(0,255)
  'Start the SD card driver
  sd.start
  'init numbers
  num.init

  'First, auto-connect at 115200
  lcd.DrawString(4,0,0,  string("Connecting..."),foregroundcolor,backgroundcolor)
  'Start the serial driver 
  ser.start(|<C328_TX, |<C328_RX,  115200)
  ser.rxflush  
  'connect to camera
  if (CheckSync)
    lcd.DrawString(4,0,0,string("Connect @115200"),foregroundcolor,backgroundcolor) 
  else
    repeat
      lcd.DrawString(4,0,1,string("Attempt#"),foregroundcolor,backgroundcolor)
      lcd.DrawString(4,9,1,num.ToStr(++i,num#dec),foregroundcolor,backgroundcolor) 
      ser.stop
      delay_ms(10)
      ser.start(|<C328_TX, |<C328_RX, 115200)
      ser.rxflush
      if (CheckSync)
         lcd.DrawString(4,0,0,string("Connect @115200"),foregroundcolor,backgroundcolor)  
         quit


  'set light frequency       
  SendCommandGetAck(@Light60Hz)

  repeat 'Main loop

    'Connect at high speed and show preview pictures
    lcd.DrawString(4,0,0,  string("Connecting...    "),foregroundcolor,backgroundcolor)
    lcd.DrawString(4,0,1,  string("                 "),foregroundcolor,backgroundcolor)
    
    'Change to new high baud for preview
    SendCommandGetAck(@B460800)
    ser.start(|<C328_TX, |<C328_RX, 460800)
    ser.rxflush
    'connect to camera at higher baud
    if (CheckSync)
      lcd.DrawString(4,0,0,string("Connect @460800"),foregroundcolor,backgroundcolor) 
    else
      repeat
        lcd.DrawString(4,0,1,string("Attempt#"),foregroundcolor,backgroundcolor)
        lcd.DrawString(4,9,1,num.ToStr(++i,num#dec),foregroundcolor,backgroundcolor) 
        ser.stop
        delay_ms(10)
        ser.start(|<C328_TX, |<C328_RX, 460800)
        ser.rxflush
        if (CheckSync)
           lcd.DrawString(4,0,0,string("Connect @460800"),foregroundcolor,backgroundcolor)  
           quit


  
    'show preview until button pushed
    lcd.DrawString(4,0,1,string("               "),foregroundcolor,backgroundcolor)
    lcd.DrawString(4,0,0,string("Showing Preview"),foregroundcolor,backgroundcolor)
    i:=0

    repeat until (INA[6]==0) 'wait for front panel button push  
      TakeShowPreview(@Init16BitPreview, 80, 64)
      delay_ms(10)
      lcd.DrawString(4,0,6,  string("                 "),foregroundcolor,backgroundcolor) 
      lcd.DrawString(4,0,6,string("Loop#"),foregroundcolor,backgroundcolor)
      lcd.DrawString(4,6,6,num.ToStr(++i,num#dec),foregroundcolor,backgroundcolor)   
   
    'save picture
    lcd.DrawString(4,0,0,string("Saving JPG to SD"),foregroundcolor,backgroundcolor)
    'mount sd card
    if (\sd.mountSD<>0)
      if (\sd.mountSD<>0)
        if (\sd.mountSD<>0)
          lcd.DrawString(4,0,1,string("Failed to mount SD"),foregroundcolor,backgroundcolor)
          repeat

    'Drop back to 115200 for jpg save

      'Change to new baud
    SendCommandGetAck(@B115200)
    ser.start(|<C328_TX, |<C328_RX, 115200)
    ser.rxflush
    'connect to camera at higher baud
    if (CheckSync)
      lcd.DrawString(4,0,0,string("Connect @115200"),foregroundcolor,backgroundcolor) 
    else
      repeat
        lcd.DrawString(4,0,1,string("Attempt#"),foregroundcolor,backgroundcolor)
        lcd.DrawString(4,9,1,num.ToStr(++i,num#dec),foregroundcolor,backgroundcolor) 
        ser.stop
        delay_ms(10)
        ser.start(|<C328_TX, |<C328_RX, 115200)
        ser.rxflush
        if (CheckSync)
           lcd.DrawString(4,0,0,string("Connect @115200"),foregroundcolor,backgroundcolor)  
           quit

    'Save JPG data to SD       
    if (TakeSaveJPG(STRING("Test2.jpg"),@InitJPG640)==true)  
      lcd.DrawString(4,0,6,string("JPG saved to SD"),foregroundcolor,backgroundcolor)  
    else
      lcd.DrawString(4,0,0,string("Save to SD failed"),foregroundcolor,backgroundcolor)
      repeat

    delay_ms(100) 



PUB TakeShowPreview(pInit, x, y)|i,j
  'Get Preview and show it on LCD
  SendCommandGetAck(pInit)
  delay_ms(10) 'need to wait for serial transmission to complete
  lcd.SetPosWindow(x, y, 160, 120)  'set starting position and window for image
  lcd.WriteReg($03,$5018)  'switch to 2-byte per pixel mode  , and set scan direction
  lcd.WriteCmd($22)  'switch to gram register

  lcd.GetC328Preview
  SendCommandGetAck(@GetPreview)
  lcd.WaitCommandDone
  ser.rxflush
  SendCommand(@DonePreview)
  lcd.ResetWindow 'reset window and mode   


    
PUB TakeSaveJPG(sFilename,pInit)
  'delay_ms(100)
  ser.rxflush
  if NOT SendCommandGetAck(pInit)
    lcd.DrawString(4,0,6,string("JPG Init fail  "),foregroundcolor,backgroundcolor)
    return false
  if NOT SendCommandGetAck(@Package512)
    lcd.DrawString(4,0,6,string("JPG PackSize fail"),foregroundcolor,backgroundcolor)
    return false
  'delay_ms(1000)  'let camera get warmed up...
  'str(string("Taking picture",13))  
  if NOT SendCommandGetAck(@Snapshot)
    lcd.DrawString(4,0,6,string("JPG Snapshot fail"),foregroundcolor,backgroundcolor) 
    return false
  if NOT SendCommandGetAck(@GetPicture)
    lcd.DrawString(4,0,6,string("JPG GetPict fail"),foregroundcolor,backgroundcolor) 
    return false

  return SaveJPG(sFilename)

  
Pub SaveJPG(sFilename)|type,size, n,i, p,j
  delay_ms(10)
  if (ser.rx<>$AA)
    lcd.DrawString(4,0,6,string("Save JPG fail"),foregroundcolor,backgroundcolor) 
    return false
  if (ser.rx<>$0A)
    lcd.DrawString(4,0,6,string("Save JPG fail"),foregroundcolor,backgroundcolor) 
    return false
        
  type:=ser.rx
  size:=ser.rx+ser.rx<<8+ser.rx<<16

  sd.popen(sFilename, "w")

  p:=0  
  SendAckPackage(p)
  delay_ms(10)
  n:=0
  repeat
    ser.rx
    ser.rx
    p++ 'new package
    i:=ser.rx+ser.rx<<8
    repeat j from 0 to i-1
      PicBuf[j]:=ser.rx
    sd.pwrite(@PicBuf,i)  
    n+=i
    ser.rx
    ser.rx
    delay_ms(1)
    if (n==size) 'All done...
      SendCommand(@DonePicture)
      sd.pclose
      return true
    else
      SendAckPackage(p)
     
    
Pub SendAckPackage(nPackage)
    ser.tx($AA)
    ser.tx($0E)
    ser.tx($00)
    ser.tx($00)
    ser.tx(nPackage//256)
    ser.tx(nPackage/256)

Pub SendCommandGetAck(pCmd):bAck
   SendCommand(pCmd)
   delay_ms(10)
   bAck:=GetAck(pCmd)
   if (bAck)
     'str(string("Command Accepted",13))
   else
     'str(string("Command Failed",13))
   return bAck  

PUB CheckSync:bSyncd 
  ser.rxflush
  repeat 60
    delay_ms(10)
    SendCommand(@Sync)'send_sync  
    delay_ms(10)
    if (GetAck(@Sync)==true)
      delay_ms(10)  
      if (GetSync==true)
        delay_ms(10)
        SendAck(@Sync)
        return true
  return false

PUB SendAck(pCmd)
  ser.tx($AA)
  ser.tx($0E)
  ser.tx(byte[pCmd])
  ser.tx(0)
  ser.tx(0)
  ser.tx(0) 

PUB GetAck(pCmd):bGot|i
  repeat 100
    if (ser.rxcheck==$AA)
      if (ser.rxcheck==$0E)
        if (ser.rxcheck==byte[pCmd])'$0D)
          if (ser.rxcheck>0)
            if (ser.rxcheck==0)
              if (ser.rxcheck==0)
                return true
  return false
                
PUB GetSync:bGot
  repeat 100
    if (ser.rxcheck==$AA)
      if (ser.rxcheck==$0D)
          if (ser.rxcheck==$00)
              if (ser.rxcheck==$00)
                  if (ser.rxcheck==$00)
                      if (ser.rxcheck==$00)
                        return true
  return false  
                   

PRI SendCommand(pCmd)|i
  ser.tx($AA)
  repeat i from 0 to 4
    ser.tx(byte[pCmd+i])

PRI delay_ms(Duration)
  waitcnt(((clkfreq / 1_000 * Duration - 3932)) + cnt)







        
DAT
Sync byte $0D,$00,$00,$00,$00
Ack  byte $0E,$0D,$00,$00,$00
DataAck byte $00                             'respond to data present
B115200 byte $07,$0F,$01,$00,$00
B14400 byte $07,$7F,$01,$00,$00
B230400 byte  $07,$07,$01,$00,$00
B460800 byte  $07,$03,$01,$00,$00
B614400 byte  $07,$02,$01,$00,$00 
B921600 byte  $07,$01,$01,$00,$00 
B153600 byte $07,11,$01,$00,$00 
Init2BitPreview byte $01,$00,$01,$03,$00     'Initialize to 2-bit grayscale 160x120
'Init16BitPreview byte $01,$00,$06,$01,$00     'Initialize to 16-bit color 80x60
Init16BitPreview byte $01,$00,$06,$03,$00     'Initialize to 16-bit color 160x120 
InitJPG640 byte $01,$00,$07,$00,$07          'Initialize to 640x480 jpg
Package512 byte $06,$08,$00,$02,$00          'Set data package size to 512 bytes
Reset byte $08,$00,$00,$00,$00
SnapShot byte $05,$00,$0A,$00,$00            'Take compressed picture  'skip a few frames
GetPicture byte $04,$01,$00,$00,$00          'Command to start sending picture data
GetPreview byte $04,$02,$00,$00,$00
DonePicture byte $0E,$00,$00,$F0,$F0
DonePreview byte $0E,$0A,$00,$00,$00
Light60Hz byte $13,$01,$00,$00,$00


 